@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -35,6 +41,7 @@
 #include <private/qsgmaterialshader_p.h>
 
 #include <qopenglshaderprogram.h>
+#include <qopenglframebufferobject.h>
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformintegration.h>
@@ -43,6 +50,7 @@
 
 #include <QtQuick/qquickwindow.h>
 #include <QtQuick/private/qsgtexture_p.h>
+#include <QtQuick/private/qsgdefaultrendercontext_p.h>
 
 #include <private/qrawfont_p.h>
 #include <QtCore/qmath.h>
@@ -73,7 +81,7 @@ static inline int qsg_device_pixel_ratio(QOpenGLContext *ctx)
         else
             devicePixelRatio = w->devicePixelRatio();
     } else {
-        devicePixelRatio = ctx->screen()->devicePixelRatio();
+        devicePixelRatio = ctx->screen() ? ctx->screen()->devicePixelRatio() : qGuiApp->devicePixelRatio();
     }
     return devicePixelRatio;
 }
@@ -92,6 +100,7 @@ protected:
     int m_matrix_id;
     int m_color_id;
     int m_textureScale_id;
+    float m_devicePixelRatio;
 
     QFontEngine::GlyphFormat m_glyphFormat;
 };
@@ -103,11 +112,14 @@ char const *const *QSGTextMaskShader::attributeNames() const
 }
 
 QSGTextMaskShader::QSGTextMaskShader(QFontEngine::GlyphFormat glyphFormat)
-    : QSGMaterialShader(*new QSGMaterialShaderPrivate),
-      m_glyphFormat(glyphFormat)
+    : QSGMaterialShader(*new QSGMaterialShaderPrivate)
+    , m_matrix_id(-1)
+    , m_color_id(-1)
+    , m_textureScale_id(-1)
+    , m_glyphFormat(glyphFormat)
 {
-    setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/scenegraph/shaders/textmask.vert"));
-    setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/textmask.frag"));
+    setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/qt-project.org/scenegraph/shaders/textmask.vert"));
+    setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/textmask.frag"));
 }
 
 static inline qreal fontSmoothingGamma()
@@ -121,7 +133,8 @@ void QSGTextMaskShader::initialize()
     m_matrix_id = program()->uniformLocation("matrix");
     m_color_id = program()->uniformLocation("color");
     m_textureScale_id = program()->uniformLocation("textureScale");
-    program()->setUniformValue("dpr", (float) qsg_device_pixel_ratio(QOpenGLContext::currentContext()));
+    m_devicePixelRatio = (float) qsg_device_pixel_ratio(QOpenGLContext::currentContext());
+    program()->setUniformValue("dpr", m_devicePixelRatio);
 }
 
 void QSGTextMaskShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
@@ -149,6 +162,12 @@ void QSGTextMaskShader::updateState(const RenderState &state, QSGMaterial *newEf
         }
     }
 
+    float devicePixelRatio = (float) qsg_device_pixel_ratio(QOpenGLContext::currentContext());
+    if (m_devicePixelRatio != devicePixelRatio) {
+        m_devicePixelRatio = devicePixelRatio;
+        program()->setUniformValue("dpr", m_devicePixelRatio);
+    }
+
     if (state.isMatrixDirty())
         program()->setUniformValue(m_matrix_id, state.combinedMatrix());
 }
@@ -159,7 +178,7 @@ public:
     QSG8BitTextMaskShader(QFontEngine::GlyphFormat glyphFormat)
         : QSGTextMaskShader(glyphFormat)
     {
-        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/8bittextmask.frag"));
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/8bittextmask.frag"));
     }
 
     virtual void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect);
@@ -184,7 +203,7 @@ public:
         : QSGTextMaskShader(glyphFormat)
         , m_useSRGB(false)
     {
-        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/24bittextmask.frag"));
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/24bittextmask.frag"));
     }
 
     virtual void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect);
@@ -192,6 +211,7 @@ public:
     void activate();
     void deactivate();
 
+    bool useSRGB() const;
     uint m_useSRGB : 1;
 };
 
@@ -211,11 +231,27 @@ void QSG24BitTextMaskShader::initialize()
     }
 }
 
+bool QSG24BitTextMaskShader::useSRGB() const
+{
+#ifdef Q_OS_MACOS
+    if (!m_useSRGB)
+        return false;
+
+    // m_useSRGB is true, but if some QOGLFBO was bound check it's texture format:
+    QOpenGLContext *ctx = QOpenGLContext::currentContext();
+    QOpenGLFramebufferObject *qfbo = QOpenGLContextPrivate::get(ctx)->qgl_current_fbo;
+    bool fboInvalid = QOpenGLContextPrivate::get(ctx)->qgl_current_fbo_invalid;
+    return !qfbo || fboInvalid || qfbo->format().internalTextureFormat() == GL_SRGB8_ALPHA8_EXT;
+#else
+    return m_useSRGB;
+#endif
+}
+
 void QSG24BitTextMaskShader::activate()
 {
     QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
     funcs->glBlendFunc(GL_CONSTANT_COLOR, GL_ONE_MINUS_SRC_COLOR);
-    if (m_useSRGB)
+    if (useSRGB())
         funcs->glEnable(GL_FRAMEBUFFER_SRGB);
 }
 
@@ -223,7 +259,7 @@ void QSG24BitTextMaskShader::deactivate()
 {
     QOpenGLFunctions *funcs = QOpenGLContext::currentContext()->functions();
     funcs->glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-    if (m_useSRGB)
+    if (useSRGB())
         funcs->glDisable(GL_FRAMEBUFFER_SRGB);
 }
 
@@ -248,7 +284,7 @@ void QSG24BitTextMaskShader::updateState(const RenderState &state, QSGMaterial *
 
     if (oldMaterial == 0 || material->color() != oldMaterial->color() || state.isOpacityDirty()) {
         QVector4D color = material->color();
-        if (m_useSRGB)
+        if (useSRGB())
             color = qt_sRGB_to_linear_RGB(color);
         QOpenGLContext::currentContext()->functions()->glBlendColor(color.x(), color.y(), color.z(), color.w());
         color = qsg_premultiply(color, state.opacity());
@@ -256,14 +292,38 @@ void QSG24BitTextMaskShader::updateState(const RenderState &state, QSGMaterial *
     }
 }
 
+class QSG32BitColorTextShader : public QSGTextMaskShader
+{
+public:
+    QSG32BitColorTextShader(QFontEngine::GlyphFormat glyphFormat)
+        : QSGTextMaskShader(glyphFormat)
+    {
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/32bitcolortext.frag"));
+    }
+
+    void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect) Q_DECL_OVERRIDE;
+};
+
+void QSG32BitColorTextShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
+{
+    QSGTextMaskShader::updateState(state, newEffect, oldEffect);
+    QSGTextMaskMaterial *material = static_cast<QSGTextMaskMaterial *>(newEffect);
+    QSGTextMaskMaterial *oldMaterial = static_cast<QSGTextMaskMaterial *>(oldEffect);
+
+    if (oldMaterial == Q_NULLPTR || material->color() != oldMaterial->color() || state.isOpacityDirty()) {
+        float opacity = material->color().w() * state.opacity();
+        program()->setUniformValue(m_color_id, opacity);
+    }
+}
+
 class QSGStyledTextShader : public QSG8BitTextMaskShader
 {
 public:
     QSGStyledTextShader(QFontEngine::GlyphFormat glyphFormat)
         : QSG8BitTextMaskShader(glyphFormat)
     {
-        setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/scenegraph/shaders/styledtext.vert"));
-        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/styledtext.frag"));
+        setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/qt-project.org/scenegraph/shaders/styledtext.vert"));
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/styledtext.frag"));
     }
 
     virtual void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect);
@@ -334,8 +394,8 @@ public:
     QSGOutlinedTextShader(QFontEngine::GlyphFormat glyphFormat)
         : QSGStyledTextShader(glyphFormat)
     {
-        setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/scenegraph/shaders/outlinedtext.vert"));
-        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/outlinedtext.frag"));
+        setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/qt-project.org/scenegraph/shaders/outlinedtext.vert"));
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/scenegraph/shaders/outlinedtext.frag"));
     }
 };
 
@@ -386,7 +446,7 @@ void QSGTextMaskMaterial::init(QFontEngine::GlyphFormat glyphFormat)
         if (!m_glyphCache || int(m_glyphCache->glyphFormat()) != glyphFormat) {
             m_glyphCache = new QOpenGLTextureGlyphCache(glyphFormat, glyphCacheTransform);
             fontEngine->setGlyphCache(ctx, m_glyphCache.data());
-            QSGRenderContext *sg = QSGRenderContext::from(ctx);
+            auto sg = QSGDefaultRenderContext::from(ctx);
             Q_ASSERT(sg);
             sg->registerFontengineForCleanup(fontEngine);
         }
@@ -403,7 +463,9 @@ void QSGTextMaskMaterial::populate(const QPointF &p,
 {
     Q_ASSERT(m_font.isValid());
     QVector<QFixedPoint> fixedPointPositions;
-    for (int i=0; i<glyphPositions.size(); ++i)
+    const int glyphPositionsSize = glyphPositions.size();
+    fixedPointPositions.reserve(glyphPositionsSize);
+    for (int i=0; i < glyphPositionsSize; ++i)
         fixedPointPositions.append(QFixedPoint::fromPointF(glyphPositions.at(i)));
 
     QTextureGlyphCache *cache = glyphCache();
@@ -484,8 +546,16 @@ void QSGTextMaskMaterial::populate(const QPointF &p,
 
 QSGMaterialType *QSGTextMaskMaterial::type() const
 {
-    static QSGMaterialType rgb, gray;
-    return glyphCache()->glyphFormat() == QFontEngine::Format_A32 ? &rgb : &gray;
+    static QSGMaterialType argb, rgb, gray;
+    switch (glyphCache()->glyphFormat()) {
+    case QFontEngine::Format_ARGB:
+        return &argb;
+    case QFontEngine::Format_A32:
+        return &rgb;
+    case QFontEngine::Format_A8:
+    default:
+        return &gray;
+    }
 }
 
 QOpenGLTextureGlyphCache *QSGTextMaskMaterial::glyphCache() const
@@ -495,10 +565,15 @@ QOpenGLTextureGlyphCache *QSGTextMaskMaterial::glyphCache() const
 
 QSGMaterialShader *QSGTextMaskMaterial::createShader() const
 {
-    QFontEngine::GlyphFormat glyphFormat = glyphCache()->glyphFormat();
-    return glyphFormat == QFontEngine::Format_A32
-           ? (QSGMaterialShader *) new QSG24BitTextMaskShader(glyphFormat)
-           : (QSGMaterialShader *) new QSG8BitTextMaskShader(glyphFormat);
+    switch (QFontEngine::GlyphFormat glyphFormat = glyphCache()->glyphFormat()) {
+    case QFontEngine::Format_ARGB:
+        return new QSG32BitColorTextShader(glyphFormat);
+    case QFontEngine::Format_A32:
+        return new QSG24BitTextMaskShader(glyphFormat);
+    case QFontEngine::Format_A8:
+    default:
+        return new QSG8BitTextMaskShader(glyphFormat);
+    }
 }
 
 static inline int qsg_colorDiff(const QVector4D &a, const QVector4D &b)
