@@ -4,22 +4,23 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.6/7.6.1/7.6.1-6-6.js
- * @description Allow reserved words as property names by dot operator assignment, accessed via indexing: continue, for, switch
+ * @path ch07/7.6/7.6.1/7.6.1-5-1.js
+ * @description Allow reserved words as property names at object initialization, accessed via indexing: null, true, false
  */
 
 
 function testcase() {
-        var tokenCodes  = {};
-        tokenCodes.continue = 0;
-        tokenCodes.for = 1; 
-        tokenCodes.switch = 2;
+        var tokenCodes  = { 
+            null: 0,
+            true: 1,
+            false: 2
+        };
         var arr = [
-            'continue', 
-            'for',
-            'switch'
-         ];
-         for (var i = 0; i < arr.length; i++) {
+            'null',
+            'true',
+            'false'
+        ];  
+        for (var i = 0; i < arr.length; i++) {
             if (tokenCodes[arr[i]] !== i) {
                 return false;
             };
