@@ -4,15 +4,15 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.3/15.3.5/15.3.5.4/15.3.5.4_2-65gs.js
- * @description Strict mode - checking access to strict function caller from non-strict function (strict function declaration called by Function.prototype.call())
+ * @path ch15/15.3/15.3.5/15.3.5.4/15.3.5.4_2-49gs.js
+ * @description Strict mode - checking access to strict function caller from non-strict function (Literal getter includes strict directive prologue)
  * @noStrict
  * @negative TypeError
  */
 
 
-function f() { "use strict"; return gNonStrict();};
-f.call();
+var o = { get foo() { "use strict"; return gNonStrict(); } }
+o.foo;
 
 
 function gNonStrict() {
