@@ -4,14 +4,14 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-7-c-i-29.js
- * @description Array.prototype.every - element changed by getter on previous iterations is observed on an Array-like object
+ * @path ch15/15.4/15.4.4/15.4.4.20/15.4.4.20-9-c-i-29.js
+ * @description Array.prototype.filter - element changed by getter on previous iterations is observed on an Array-like object
  */
 
 
 function testcase() {
         function callbackfn(val, idx, obj) {
-            return val > 10;
+            return val === 9 && idx === 1;
         }
 
         var preIterVisible = false;
@@ -35,7 +35,8 @@ function testcase() {
             },
             configurable: true
         });
+        var newArr = Array.prototype.filter.call(obj, callbackfn);
 
-        return !Array.prototype.every.call(obj, callbackfn);
+        return newArr.length === 1 && newArr[0] === 9;
     }
 runTestCase(testcase);
