@@ -4,8 +4,8 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.19/15.4.4.19-8-c-ii-22.js
- * @description Array.prototype.map - callbackfn called with correct parameters (the index k is correct)
+ * @path ch15/15.4/15.4.4/15.4.4.20/15.4.4.20-9-c-ii-22.js
+ * @description Array.prototype.filter - callbackfn called with correct parameters (the index k is correct)
  */
 
 
@@ -24,9 +24,8 @@ function testcase() {
         }
 
         var obj = { 0: 11, 1: 12, length: 2 };
+        var newArr = Array.prototype.filter.call(obj, callbackfn);
 
-        var testResult = Array.prototype.map.call(obj, callbackfn);
-
-        return testResult[0] === true && testResult[1] === true;
+        return newArr.length === 2 && newArr[0] === 11 && newArr[1] === 12;
     }
 runTestCase(testcase);
