@@ -4,21 +4,22 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.22/15.4.4.22-9-c-i-11.js
- * @description Array.prototype.reduceRight - element to be retrieved is own accessor property that overrides an inherited data property on an Array-like object
+ * @path ch15/15.4/15.4.4/15.4.4.21/15.4.4.21-9-c-i-11.js
+ * @description Array.prototype.reduce - element to be retrieved is own accessor property that overrides an inherited data property on an Array-like object
  */
 
 
 function testcase() {
 
         var testResult = false;
+        var initialValue = 0;
         function callbackfn(prevVal, curVal, idx, obj) {
             if (idx === 1) {
-                testResult = (curVal === "1");
+                testResult = (curVal === "11");
             }
         }
 
-        var proto = { 0: 0, 1: 11, 2: 2 };
+        var proto = { 0: 0, 1: 1, 2: 2 };
 
         var Con = function () { };
         Con.prototype = proto;
@@ -28,12 +29,12 @@ function testcase() {
 
         Object.defineProperty(child, "1", {
             get: function () {
-                return "1";
+                return "11";
             },
             configurable: true
         });
 
-        Array.prototype.reduceRight.call(child, callbackfn, "initialValue");
+        Array.prototype.reduce.call(child, callbackfn, initialValue);
         return testResult;
     }
 runTestCase(testcase);
