@@ -4,21 +4,12 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.4/15.4.4/15.4.4.16/15.4.4.16-4-7.js
- * @description Array.prototype.every throws TypeError if callbackfn is Object without a Call internal method
+ * @path ch15/15.5/15.5.4/15.5.4.20/15.5.4.20-2-37.js
+ * @description String.prototype.trim - 'this' is a Number Object that converts to a string
  */
 
 
 function testcase() {
-
-  var arr = new Array(10);
-  try {
-    arr.every( {} );    
-  }
-  catch(e) {
-    if(e instanceof TypeError)
-      return true;  
-  }
-
- }
+        return (String.prototype.trim.call(new Number(123)) === "123");
+    }
 runTestCase(testcase);
